@@ -10,8 +10,8 @@ from specfile.specfile import Specfile
 try:
     from importlib.metadata import PackageNotFoundError, distribution
 except ImportError:
-    from importlib_metadata import PackageNotFoundError  # type: ignore
-    from importlib_metadata import distribution  # type: ignore
+    from importlib_metadata import PackageNotFoundError  # type: ignore[no-redef]
+    from importlib_metadata import distribution  # type: ignore[no-redef]
 
 try:
     __version__ = distribution(__name__).version
