@@ -3,13 +3,28 @@
 
 import collections
 import datetime
+import re
 from typing import List, Optional, Union, overload
 
-import arrow
-
 from specfile.sections import Section
 from specfile.types import SupportsIndex
 
+WEEKDAYS = ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
+MONTHS = (
+    "Jan",
+    "Feb",
+    "Mar",
+    "Apr",
+    "May",
+    "Jun",
+    "Jul",
+    "Aug",
+    "Sep",
+    "Oct",
+    "Nov",
+    "Dec",
+)
+
 
 class ChangelogEntry:
     """
@@ -54,12 +69,51 @@ class ChangelogEntry:
     @property
     def extended_timestamp(self) -> bool:
         """Whether the timestamp present in the entry header is extended (date and time)."""
-        try:
-            arrow.get(self.header, "ddd MMM DD YYYY")
-        except arrow.parser.ParserError:
-            return True
-        else:
-            return False
+        weekdays = "|".join(WEEKDAYS)
+        months = "|".join(MONTHS)
+        m = re.search(
+            rf"""
+            ({weekdays})     # weekday
+            [ ]
+            ({months})       # month
+            [ ]+
+            ([12]?\d|3[01])  # day of month
+            [ ]
+            ([01]\d|2[0-3])  # hour
+            :
+            ([0-5]\d)        # minute
+            :
+            ([0-5]\d)        # second
+            [ ]
+            \S+              # timezone
+            [ ]
+            \d{{4}}          # year
+            """,
+            self.header,
+            re.VERBOSE,
+        )
+        return m is not None
+
+    @property
+    def day_of_month_padding(self) -> str:
+        """Padding of day of month in the entry header timestamp"""
+        weekdays = "|".join(WEEKDAYS)
+        months = "|".join(MONTHS)
+        m = re.search(
+            rf"""
+            ({weekdays})                 # weekday
+            [ ]
+            ({months})                   # month
+            [ ]
+            (?P<wsp>[ ]*)                # optional whitespace padding
+            ((?P<zp>0)?\d|[12]\d|3[01])  # possibly zero-padded day of month
+            """,
+            self.header,
+            re.VERBOSE,
+        )
+        if not m:
+            return ""
+        return m.group("wsp") + (m.group("zp") or "")
 
     @staticmethod
     def assemble(
@@ -67,6 +121,7 @@ class ChangelogEntry:
         author: str,
         content: List[str],
         evr: Optional[str] = None,
+        day_of_month_padding: str = "0",
         append_newline: bool = True,
     ) -> "ChangelogEntry":
         """
@@ -78,20 +133,25 @@ class ChangelogEntry:
             author: Author of the entry.
             content: List of lines forming the content of the entry.
             evr: EVR (epoch, version, release) of the entry.
+            day_of_month_padding: Padding to apply to day of month in the timestamp.
             append_newline: Whether the entry should be followed by an empty line.
 
         Returns:
             Constructed instance of `ChangelogEntry` class.
         """
-        header = "*"
+        weekday = WEEKDAYS[timestamp.weekday()]
+        month = MONTHS[timestamp.month - 1]
+        header = f"* {weekday} {month}"
+        if day_of_month_padding.endswith("0"):
+            header += f" {day_of_month_padding[:-1]}{timestamp.day:02}"
+        else:
+            header += f" {day_of_month_padding}{timestamp.day}"
         if isinstance(timestamp, datetime.datetime):
             # extended format
-            header += arrow.Arrow.fromdatetime(timestamp).format(
-                " ddd MMM DD hh:mm:ss ZZZ YYYY"
-            )
-        else:
-            header += arrow.Arrow.fromdate(timestamp).format(" ddd MMM DD YYYY")
-        header += f" {author}"
+            if not timestamp.tzinfo:
+                timestamp = timestamp.replace(tzinfo=datetime.timezone.utc)
+            header += f" {timestamp:%H:%M:%S %Z}"
+        header += f" {timestamp:%Y} {author}"
         if evr is not None:
             header += f" - {evr}"
         return ChangelogEntry(header, content, [""] if append_newline else None)
