@@ -3,12 +3,51 @@
 
 import datetime
 
-import dateutil.tz
+import pytest
 
 from specfile.changelog import Changelog, ChangelogEntry
 from specfile.sections import Section
 
 
+@pytest.mark.parametrize(
+    "header, extended",
+    [
+        ("* Tue May 4 2021 Nikola Forró <nforro@redhat.com> - 0.1-1", False),
+        ("* Tue May  4 2021 Nikola Forró <nforro@redhat.com> - 0.1-1", False),
+        (
+            "* Thu Jul 22 2021 Fedora Release Engineering <releng@fedoraproject.org> - 0.1-2",
+            False,
+        ),
+        (
+            "* Mon Oct 18 12:34:45 CEST 2021 Nikola Forró <nforro@redhat.com> - 0.2-1",
+            True,
+        ),
+    ],
+)
+def test_entry_has_extended_timestamp(header, extended):
+    assert ChangelogEntry(header, [""]).extended_timestamp == extended
+
+
+@pytest.mark.parametrize(
+    "header, padding",
+    [
+        ("* Tue May 4 2021 Nikola Forró <nforro@redhat.com> - 0.1-1", ""),
+        ("* Tue May 04 2021 Nikola Forró <nforro@redhat.com> - 0.1-1", "0"),
+        ("* Tue May  4 2021 Nikola Forró <nforro@redhat.com> - 0.1-1", " "),
+        (
+            "* Thu Jul 22 2021 Fedora Release Engineering <releng@fedoraproject.org> - 0.1-2",
+            "",
+        ),
+        (
+            "* Mon Oct  18 12:34:45 CEST 2021 Nikola Forró <nforro@redhat.com> - 0.2-1",
+            " ",
+        ),
+    ],
+)
+def test_entry_day_of_month_padding(header, padding):
+    assert ChangelogEntry(header, [""]).day_of_month_padding == padding
+
+
 def test_parse():
     changelog = Changelog.parse(
         Section(
@@ -59,6 +98,7 @@ def test_parse():
 
 
 def test_get_raw_section_data():
+    tzinfo = datetime.timezone(datetime.timedelta(hours=2), name="CEST")
     changelog = Changelog(
         [
             ChangelogEntry.assemble(
@@ -75,9 +115,7 @@ def test_get_raw_section_data():
                 "0.1-2",
             ),
             ChangelogEntry.assemble(
-                datetime.datetime(
-                    2021, 10, 18, 12, 34, 45, tzinfo=dateutil.tz.gettz("CET")
-                ),
+                datetime.datetime(2021, 10, 18, 12, 34, 45, tzinfo=tzinfo),
                 "Nikola Forró <nforro@redhat.com>",
                 ["- new upstream release"],
                 "0.2-1",
